package org.joda.money;

import java.math.BigDecimal;
import java.math.BigInteger;

import org.testng.annotations.Test;

import junit.framework.TestCase;

public class TestBigMoney1 extends TestCase {

	private static final CurrencyUnit GBP = CurrencyUnit.of("GBP");
	private static final CurrencyUnit EUR = CurrencyUnit.of("EUR");
	private static final CurrencyUnit USD = CurrencyUnit.of("USD");
	private static final CurrencyUnit JPY = CurrencyUnit.of("JPY");
	private static final BigDecimal BIGDEC_2_34 = new BigDecimal("2.34");
	private static final BigDecimal BIGDEC_2_345 = new BigDecimal("2.345");
	private static final BigDecimal BIGDEC_M5_78 = new BigDecimal("-5.78");

	private static final BigMoney GBP_0_00 = BigMoney.parse("GBP 0.00");
	private static final BigMoney GBP_1_23 = BigMoney.parse("GBP 1.23");
	private static final BigMoney GBP_2_33 = BigMoney.parse("GBP 2.33");
	private static final BigMoney GBP_2_34 = BigMoney.parse("GBP 2.34");
	private static final BigMoney GBP_2_35 = BigMoney.parse("GBP 2.35");
	private static final BigMoney GBP_2_36 = BigMoney.parse("GBP 2.36");
	private static final BigMoney GBP_5_78 = BigMoney.parse("GBP 5.78");
	private static final BigMoney GBP_M1_23 = BigMoney.parse("GBP -1.23");
	private static final BigMoney GBP_M5_78 = BigMoney.parse("GBP -5.78");
	private static final BigMoney GBP_INT_MAX_PLUS1 = BigMoney.ofMinor(GBP, ((long) Integer.MAX_VALUE) + 1);
	private static final BigMoney GBP_INT_MIN_MINUS1 = BigMoney.ofMinor(GBP, ((long) Integer.MIN_VALUE) - 1);
	private static final BigMoney GBP_INT_MAX_MAJOR_PLUS1 = BigMoney.ofMinor(GBP,
			(((long) Integer.MAX_VALUE) + 1) * 100);
	private static final BigMoney GBP_INT_MIN_MAJOR_MINUS1 = BigMoney.ofMinor(GBP,
			(((long) Integer.MIN_VALUE) - 1) * 100);
	private static final BigMoney GBP_LONG_MAX_PLUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE));
	private static final BigMoney GBP_LONG_MIN_MINUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE));
	private static final BigMoney GBP_LONG_MAX_MAJOR_PLUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE).multiply(BigDecimal.valueOf(100)));
	private static final BigMoney GBP_LONG_MIN_MAJOR_MINUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE).multiply(BigDecimal.valueOf(100)));
	private static final BigMoney JPY_423 = BigMoney.parse("JPY 423");
	private static final BigMoney USD_1_23 = BigMoney.parse("USD 1.23");
	private static final BigMoney USD_2_34 = BigMoney.parse("USD 2.34");
	private static final BigMoney USD_2_35 = BigMoney.parse("USD 2.35");
	private static final BigMoneyProvider BAD_PROVIDER = new BigMoneyProvider() {
		@Override
		public BigMoney toBigMoney() {
			return null; // shouldn't return null
		}
	};

	private static BigDecimal bd(String str) {
		return new BigDecimal(str);
	}

	// -----------------------------------------------------------------------
	// of(Currency,BigDecimal)
	// -----------------------------------------------------------------------
	public void test_factory_of_Currency_BigDecimal() {
		BigMoney test = BigMoney.of(GBP, BIGDEC_2_345);
	}

	@Test(expectedExceptions = NullPointerException.class)
	public void test_factory_of_Currency_BigDecimal_nullCurrency() {
		BigMoney.of((CurrencyUnit) null, BIGDEC_2_345);
	}

	public void test_factory_of_Currency_BigDecimal_nullBigDecimal() {
		BigMoney.of(GBP, (BigDecimal) null);
	}

	public void test_factory_of_Currency_subClass1() {
		class BadDecimal extends BigDecimal {
			private static final long serialVersionUID = 1L;

			BadDecimal() {
				super(432);
			}

			@Override
			public BigInteger unscaledValue() {
				return null;
			}

			@Override
			public int scale() {
				return 1;
			}
		}
		BigDecimal sub = new BadDecimal();
		BigMoney.of(GBP, sub);
	}

	public void test_factory_of_Currency_subClass2() {
		class BadInteger extends BigInteger {
			private static final long serialVersionUID = 1L;

			public BadInteger() {
				super("123");
			}
		}
		class BadDecimal extends BigDecimal {
			private static final long serialVersionUID = 1L;

			BadDecimal() {
				super(432);
			}

			@Override
			public BigInteger unscaledValue() {
				return new BadInteger();
			}

			@Override
			public int scale() {
				return 1;
			}
		}
		BigDecimal sub = new BadDecimal();
		BigMoney test = BigMoney.of(GBP, sub);
	}
	
	public static void main(String[] args) {
		TestBigMoney1 t = new TestBigMoney1();
		t.test_factory_of_Currency_BigDecimal();
		t.test_factory_of_Currency_subClass2();
	}

}
