package org.joda.money;

import java.math.BigDecimal;

import junit.framework.TestCase;

public class TestBigMoney11 extends TestCase {

	private static final CurrencyUnit GBP = CurrencyUnit.of("GBP");
	private static final CurrencyUnit EUR = CurrencyUnit.of("EUR");
	private static final CurrencyUnit USD = CurrencyUnit.of("USD");
	private static final CurrencyUnit JPY = CurrencyUnit.of("JPY");
	private static final BigDecimal BIGDEC_2_34 = new BigDecimal("2.34");
	private static final BigDecimal BIGDEC_2_345 = new BigDecimal("2.345");
	private static final BigDecimal BIGDEC_M5_78 = new BigDecimal("-5.78");

	private static final BigMoney GBP_0_00 = BigMoney.parse("GBP 0.00");
	private static final BigMoney GBP_1_23 = BigMoney.parse("GBP 1.23");
	private static final BigMoney GBP_2_33 = BigMoney.parse("GBP 2.33");
	private static final BigMoney GBP_2_34 = BigMoney.parse("GBP 2.34");
	private static final BigMoney GBP_2_35 = BigMoney.parse("GBP 2.35");
	private static final BigMoney GBP_2_36 = BigMoney.parse("GBP 2.36");
	private static final BigMoney GBP_5_78 = BigMoney.parse("GBP 5.78");
	private static final BigMoney GBP_M1_23 = BigMoney.parse("GBP -1.23");
	private static final BigMoney GBP_M5_78 = BigMoney.parse("GBP -5.78");
	private static final BigMoney GBP_INT_MAX_PLUS1 = BigMoney.ofMinor(GBP, ((long) Integer.MAX_VALUE) + 1);
	private static final BigMoney GBP_INT_MIN_MINUS1 = BigMoney.ofMinor(GBP, ((long) Integer.MIN_VALUE) - 1);
	private static final BigMoney GBP_INT_MAX_MAJOR_PLUS1 = BigMoney.ofMinor(GBP,
			(((long) Integer.MAX_VALUE) + 1) * 100);
	private static final BigMoney GBP_INT_MIN_MAJOR_MINUS1 = BigMoney.ofMinor(GBP,
			(((long) Integer.MIN_VALUE) - 1) * 100);
	private static final BigMoney GBP_LONG_MAX_PLUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE));
	private static final BigMoney GBP_LONG_MIN_MINUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE));
	private static final BigMoney GBP_LONG_MAX_MAJOR_PLUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE).multiply(BigDecimal.valueOf(100)));
	private static final BigMoney GBP_LONG_MIN_MAJOR_MINUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE).multiply(BigDecimal.valueOf(100)));
	private static final BigMoney JPY_423 = BigMoney.parse("JPY 423");
	private static final BigMoney USD_1_23 = BigMoney.parse("USD 1.23");
	private static final BigMoney USD_2_34 = BigMoney.parse("USD 2.34");
	private static final BigMoney USD_2_35 = BigMoney.parse("USD 2.35");
	private static final BigMoneyProvider BAD_PROVIDER = new BigMoneyProvider() {
		@Override
		public BigMoney toBigMoney() {
			return null; // shouldn't return null
		}
	};

	private static BigDecimal bd(String str) {
		return new BigDecimal(str);
	}

	// -----------------------------------------------------------------------
	// total(BigMoneyProvider...)
	// -----------------------------------------------------------------------
	public void test_factory_total_varargs_1BigMoney() {
		BigMoney test = BigMoney.total(GBP_1_23);
	}

	public void test_factory_total_array_1BigMoney() {
		BigMoneyProvider[] array = new BigMoneyProvider[] { GBP_1_23 };
		BigMoney test = BigMoney.total(array);
	}

	public void test_factory_total_varargs_3Mixed() {
		BigMoney test = BigMoney.total(GBP_1_23, GBP_2_33.toMoney(), GBP_2_36);
	}

	public void test_factory_total_array_3Mixed() {
		BigMoneyProvider[] array = new BigMoneyProvider[] { GBP_1_23, GBP_2_33.toMoney(), GBP_2_36 };
		BigMoney test = BigMoney.total(array);
	}

	public void test_factory_total_array_3Money() {
		Money[] array = new Money[] { GBP_1_23.toMoney(), GBP_2_33.toMoney(), GBP_2_36.toMoney() };
		BigMoney test = BigMoney.total(array);
	}

	public void test_factory_total_varargs_empty() {
		BigMoney.total();
	}

	public void test_factory_total_array_empty() {
		BigMoneyProvider[] array = new BigMoneyProvider[0];
		BigMoney.total(array);
	}

	public void test_factory_total_varargs_currenciesDiffer() {
		BigMoney.total(GBP_2_33, JPY_423);
	}

	public void test_factory_total_array_currenciesDiffer() {
		BigMoneyProvider[] array = new BigMoneyProvider[] { GBP_2_33, JPY_423 };
		BigMoney.total(array);
	}

	public void test_factory_total_varargs_nullFirst() {
		BigMoney.total((BigMoney) null, GBP_2_33, GBP_2_36);
	}

	public void test_factory_total_array_nullFirst() {
		BigMoneyProvider[] array = new BigMoneyProvider[] { null, GBP_2_33, GBP_2_36 };
		BigMoney.total(array);
	}

	public void test_factory_total_varargs_nullNotFirst() {
		BigMoney.total(GBP_2_33, null, GBP_2_36);
	}

	public void test_factory_total_array_nullNotFirst() {
		BigMoneyProvider[] array = new BigMoneyProvider[] { GBP_2_33, null, GBP_2_36 };
		BigMoney.total(array);
	}

	public void test_factory_total_varargs_badProvider() {
		BigMoney.total(BAD_PROVIDER);
	}

	public void test_factory_total_array_badProvider() {
		BigMoneyProvider[] array = new BigMoneyProvider[] { BAD_PROVIDER };
		BigMoney.total(array);
	}
	
	public static void main(String[] args) {
		TestBigMoney11 t = new TestBigMoney11();
		t.test_factory_total_varargs_1BigMoney();
		t.test_factory_total_array_1BigMoney();
		t.test_factory_total_varargs_3Mixed();
		t.test_factory_total_array_3Mixed();
		t.test_factory_total_array_3Money();
	}

}
