package org.joda.money;

import java.math.BigDecimal;

import junit.framework.TestCase;

public class TestBigMoney13 extends TestCase {

	private static final CurrencyUnit GBP = CurrencyUnit.of("GBP");
	private static final CurrencyUnit EUR = CurrencyUnit.of("EUR");
	private static final CurrencyUnit USD = CurrencyUnit.of("USD");
	private static final CurrencyUnit JPY = CurrencyUnit.of("JPY");
	private static final BigDecimal BIGDEC_2_34 = new BigDecimal("2.34");
	private static final BigDecimal BIGDEC_2_345 = new BigDecimal("2.345");
	private static final BigDecimal BIGDEC_M5_78 = new BigDecimal("-5.78");

	private static final BigMoney GBP_0_00 = BigMoney.parse("GBP 0.00");
	private static final BigMoney GBP_1_23 = BigMoney.parse("GBP 1.23");
	private static final BigMoney GBP_2_33 = BigMoney.parse("GBP 2.33");
	private static final BigMoney GBP_2_34 = BigMoney.parse("GBP 2.34");
	private static final BigMoney GBP_2_35 = BigMoney.parse("GBP 2.35");
	private static final BigMoney GBP_2_36 = BigMoney.parse("GBP 2.36");
	private static final BigMoney GBP_5_78 = BigMoney.parse("GBP 5.78");
	private static final BigMoney GBP_M1_23 = BigMoney.parse("GBP -1.23");
	private static final BigMoney GBP_M5_78 = BigMoney.parse("GBP -5.78");
	private static final BigMoney GBP_INT_MAX_PLUS1 = BigMoney.ofMinor(GBP, ((long) Integer.MAX_VALUE) + 1);
	private static final BigMoney GBP_INT_MIN_MINUS1 = BigMoney.ofMinor(GBP, ((long) Integer.MIN_VALUE) - 1);
	private static final BigMoney GBP_INT_MAX_MAJOR_PLUS1 = BigMoney.ofMinor(GBP,
			(((long) Integer.MAX_VALUE) + 1) * 100);
	private static final BigMoney GBP_INT_MIN_MAJOR_MINUS1 = BigMoney.ofMinor(GBP,
			(((long) Integer.MIN_VALUE) - 1) * 100);
	private static final BigMoney GBP_LONG_MAX_PLUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE));
	private static final BigMoney GBP_LONG_MIN_MINUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE));
	private static final BigMoney GBP_LONG_MAX_MAJOR_PLUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE).multiply(BigDecimal.valueOf(100)));
	private static final BigMoney GBP_LONG_MIN_MAJOR_MINUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE).multiply(BigDecimal.valueOf(100)));
	private static final BigMoney JPY_423 = BigMoney.parse("JPY 423");
	private static final BigMoney USD_1_23 = BigMoney.parse("USD 1.23");
	private static final BigMoney USD_2_34 = BigMoney.parse("USD 2.34");
	private static final BigMoney USD_2_35 = BigMoney.parse("USD 2.35");
	private static final BigMoneyProvider BAD_PROVIDER = new BigMoneyProvider() {
		@Override
		public BigMoney toBigMoney() {
			return null; // shouldn't return null
		}
	};

	private static BigDecimal bd(String str) {
		return new BigDecimal(str);
	}

	// -----------------------------------------------------------------------
	// total(CurrencyUnit,BigMoneyProvider...)
	// -----------------------------------------------------------------------
	public void test_factory_total_CurrencyUnitVarargs_1() {
		BigMoney test = BigMoney.total(GBP, GBP_1_23);
	}

	public void test_factory_total_CurrencyUnitArray_1() {
		BigMoney[] array = new BigMoney[] { GBP_1_23 };
		BigMoney test = BigMoney.total(GBP, array);
	}

	public void test_factory_total_CurrencyUnitVarargs_3() {
		BigMoney test = BigMoney.total(GBP, GBP_1_23, GBP_2_33, GBP_2_36);
	}

	public void test_factory_total_CurrencyUnitArray_3() {
		BigMoney[] array = new BigMoney[] { GBP_1_23, GBP_2_33, GBP_2_36 };
		BigMoney test = BigMoney.total(GBP, array);
	}

	public void test_factory_total_CurrencyUnitVarargs_3Mixed() {
		BigMoney test = BigMoney.total(GBP, GBP_1_23, GBP_2_33.toMoney(), GBP_2_36);
	}

	public void test_factory_total_CurrencyUnitArray_3Mixed() {
		BigMoneyProvider[] array = new BigMoneyProvider[] { GBP_1_23, GBP_2_33.toMoney(), GBP_2_36 };
		BigMoney test = BigMoney.total(GBP, array);
	}

	public void test_factory_total_CurrencyUnitArray_3Money() {
		Money[] array = new Money[] { GBP_1_23.toMoney(), GBP_2_33.toMoney(), GBP_2_36.toMoney() };
		BigMoney test = BigMoney.total(GBP, array);
	}

	public void test_factory_total_CurrencyUnitVarargs_empty() {
		BigMoney test = BigMoney.total(GBP);
	}

	public void test_factory_total_CurrencyUnitArray_empty() {
		BigMoney[] array = new BigMoney[0];
		BigMoney test = BigMoney.total(GBP, array);
	}

	public void test_factory_total_CurrencyUnitVarargs_currenciesDiffer() {
		BigMoney.total(GBP, JPY_423);
	}

	public void test_factory_total_CurrencyUnitArray_currenciesDiffer() {
		BigMoney[] array = new BigMoney[] { JPY_423 };
		BigMoney.total(GBP, array);
	}

	public void test_factory_total_CurrencyUnitVarargs_currenciesDifferInArray() {
		BigMoney.total(GBP, GBP_2_33, JPY_423);
	}

	public void test_factory_total_CurrencyUnitArray_currenciesDifferInArray() {
		BigMoney[] array = new BigMoney[] { GBP_2_33, JPY_423 };
		BigMoney.total(GBP, array);
	}

	public void test_factory_total_CurrencyUnitVarargs_nullFirst() {
		BigMoney.total(GBP, null, GBP_2_33, GBP_2_36);
	}

	public void test_factory_total_CurrencyUnitArray_nullFirst() {
		BigMoney[] array = new BigMoney[] { null, GBP_2_33, GBP_2_36 };
		BigMoney.total(GBP, array);
	}

	public void test_factory_total_CurrencyUnitVarargs_nullNotFirst() {
		BigMoney.total(GBP, GBP_2_33, null, GBP_2_36);
	}

	public void test_factory_total_CurrencyUnitArray_nullNotFirst() {
		BigMoney[] array = new BigMoney[] { GBP_2_33, null, GBP_2_36 };
		BigMoney.total(GBP, array);
	}

	public void test_factory_total_CurrencyUnitVarargs_badProvider() {
		BigMoney.total(GBP, BAD_PROVIDER);
	}

	public void test_factory_total_CurrencyUnitArray_badProvider() {
		BigMoneyProvider[] array = new BigMoneyProvider[] { BAD_PROVIDER };
		BigMoney.total(GBP, array);
	}
	
	public static void main(String[] args) {
		TestBigMoney13 t = new TestBigMoney13();
		t.test_factory_total_CurrencyUnitVarargs_1();
		t.test_factory_total_CurrencyUnitArray_1();
		t.test_factory_total_CurrencyUnitVarargs_3();
		t.test_factory_total_CurrencyUnitArray_3();
		t.test_factory_total_CurrencyUnitVarargs_3Mixed();
		t.test_factory_total_CurrencyUnitArray_3Mixed();
		t.test_factory_total_CurrencyUnitArray_3Money();
		t.test_factory_total_CurrencyUnitVarargs_empty();
		t.test_factory_total_CurrencyUnitArray_empty();
	}

}
