package org.joda.money;

import static org.testng.Assert.assertEquals;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Collections;

import org.testng.annotations.Test;

import junit.framework.TestCase;

public class TestBigMoney14 extends TestCase {

	private static final CurrencyUnit GBP = CurrencyUnit.of("GBP");
	private static final CurrencyUnit EUR = CurrencyUnit.of("EUR");
	private static final CurrencyUnit USD = CurrencyUnit.of("USD");
	private static final CurrencyUnit JPY = CurrencyUnit.of("JPY");
	private static final BigDecimal BIGDEC_2_34 = new BigDecimal("2.34");
	private static final BigDecimal BIGDEC_2_345 = new BigDecimal("2.345");
	private static final BigDecimal BIGDEC_M5_78 = new BigDecimal("-5.78");

	private static final BigMoney GBP_0_00 = BigMoney.parse("GBP 0.00");
	private static final BigMoney GBP_1_23 = BigMoney.parse("GBP 1.23");
	private static final BigMoney GBP_2_33 = BigMoney.parse("GBP 2.33");
	private static final BigMoney GBP_2_34 = BigMoney.parse("GBP 2.34");
	private static final BigMoney GBP_2_35 = BigMoney.parse("GBP 2.35");
	private static final BigMoney GBP_2_36 = BigMoney.parse("GBP 2.36");
	private static final BigMoney GBP_5_78 = BigMoney.parse("GBP 5.78");
	private static final BigMoney GBP_M1_23 = BigMoney.parse("GBP -1.23");
	private static final BigMoney GBP_M5_78 = BigMoney.parse("GBP -5.78");
	private static final BigMoney GBP_INT_MAX_PLUS1 = BigMoney.ofMinor(GBP, ((long) Integer.MAX_VALUE) + 1);
	private static final BigMoney GBP_INT_MIN_MINUS1 = BigMoney.ofMinor(GBP, ((long) Integer.MIN_VALUE) - 1);
	private static final BigMoney GBP_INT_MAX_MAJOR_PLUS1 = BigMoney.ofMinor(GBP,
			(((long) Integer.MAX_VALUE) + 1) * 100);
	private static final BigMoney GBP_INT_MIN_MAJOR_MINUS1 = BigMoney.ofMinor(GBP,
			(((long) Integer.MIN_VALUE) - 1) * 100);
	private static final BigMoney GBP_LONG_MAX_PLUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE));
	private static final BigMoney GBP_LONG_MIN_MINUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE));
	private static final BigMoney GBP_LONG_MAX_MAJOR_PLUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE).multiply(BigDecimal.valueOf(100)));
	private static final BigMoney GBP_LONG_MIN_MAJOR_MINUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE).multiply(BigDecimal.valueOf(100)));
	private static final BigMoney JPY_423 = BigMoney.parse("JPY 423");
	private static final BigMoney USD_1_23 = BigMoney.parse("USD 1.23");
	private static final BigMoney USD_2_34 = BigMoney.parse("USD 2.34");
	private static final BigMoney USD_2_35 = BigMoney.parse("USD 2.35");
	private static final BigMoneyProvider BAD_PROVIDER = new BigMoneyProvider() {
		@Override
		public BigMoney toBigMoney() {
			return null; // shouldn't return null
		}
	};

	private static BigDecimal bd(String str) {
		return new BigDecimal(str);
	}

	// -----------------------------------------------------------------------
	// total(CurrencyUnit,Iterable)
	// -----------------------------------------------------------------------
	public void test_factory_total_CurrencyUnitIterable() {
		Iterable<BigMoney> iterable = Arrays.asList(GBP_1_23, GBP_2_33, BigMoney.of(GBP, 2.361d));
		BigMoney test = BigMoney.total(GBP, iterable);
	}

	public void test_factory_total_CurrencyUnitIterable_Mixed() {
		Iterable<BigMoneyProvider> iterable = Arrays.<BigMoneyProvider> asList(GBP_1_23.toMoney(), GBP_2_33);
		BigMoney test = BigMoney.total(GBP, iterable);
	}

	public void test_factory_total_CurrencyUnitIterable_empty() {
		Iterable<BigMoney> iterable = Collections.emptyList();
		BigMoney test = BigMoney.total(GBP, iterable);
	}

	public void test_factory_total_CurrencyUnitIterable_currenciesDiffer() {
		Iterable<BigMoney> iterable = Arrays.asList(JPY_423);
		BigMoney.total(GBP, iterable);
	}

	public void test_factory_total_CurrencyUnitIterable_currenciesDifferInIterable() {
		Iterable<BigMoney> iterable = Arrays.asList(GBP_2_33, JPY_423);
		BigMoney.total(GBP, iterable);
	}

	public void test_factory_total_CurrencyUnitIterable_nullFirst() {
		Iterable<BigMoney> iterable = Arrays.asList(null, GBP_2_33, GBP_2_36);
		BigMoney.total(GBP, iterable);
	}

	public void test_factory_total_CurrencyUnitIterable_nullNotFirst() {
		Iterable<BigMoney> iterable = Arrays.asList(GBP_2_33, null, GBP_2_36);
		BigMoney.total(GBP, iterable);
	}

	public void test_factory_total_CurrencyUnitIterable_badProvider() {
		Iterable<BigMoneyProvider> iterable = Arrays.<BigMoneyProvider> asList(BAD_PROVIDER);
		BigMoney.total(GBP, iterable);
	}

	public static void main(String[] args) {
		TestBigMoney14 t = new TestBigMoney14();
		t.test_factory_total_CurrencyUnitIterable();
		t.test_factory_total_CurrencyUnitIterable_Mixed();
		t.test_factory_total_CurrencyUnitIterable_empty();
	}
	
}
