package org.joda.money;

import java.math.BigDecimal;

import junit.framework.TestCase;

public class TestBigMoney27 extends TestCase {

	private static final CurrencyUnit GBP = CurrencyUnit.of("GBP");
	private static final CurrencyUnit EUR = CurrencyUnit.of("EUR");
	private static final CurrencyUnit USD = CurrencyUnit.of("USD");
	private static final CurrencyUnit JPY = CurrencyUnit.of("JPY");
	private static final BigDecimal BIGDEC_2_34 = new BigDecimal("2.34");
	private static final BigDecimal BIGDEC_2_345 = new BigDecimal("2.345");
	private static final BigDecimal BIGDEC_M5_78 = new BigDecimal("-5.78");

	private static final BigMoney GBP_0_00 = BigMoney.parse("GBP 0.00");
	private static final BigMoney GBP_1_23 = BigMoney.parse("GBP 1.23");
	private static final BigMoney GBP_2_33 = BigMoney.parse("GBP 2.33");
	private static final BigMoney GBP_2_34 = BigMoney.parse("GBP 2.34");
	private static final BigMoney GBP_2_35 = BigMoney.parse("GBP 2.35");
	private static final BigMoney GBP_2_36 = BigMoney.parse("GBP 2.36");
	private static final BigMoney GBP_5_78 = BigMoney.parse("GBP 5.78");
	private static final BigMoney GBP_M1_23 = BigMoney.parse("GBP -1.23");
	private static final BigMoney GBP_M5_78 = BigMoney.parse("GBP -5.78");
	private static final BigMoney GBP_INT_MAX_PLUS1 = BigMoney.ofMinor(GBP, ((long) Integer.MAX_VALUE) + 1);
	private static final BigMoney GBP_INT_MIN_MINUS1 = BigMoney.ofMinor(GBP, ((long) Integer.MIN_VALUE) - 1);
	private static final BigMoney GBP_INT_MAX_MAJOR_PLUS1 = BigMoney.ofMinor(GBP,
			(((long) Integer.MAX_VALUE) + 1) * 100);
	private static final BigMoney GBP_INT_MIN_MAJOR_MINUS1 = BigMoney.ofMinor(GBP,
			(((long) Integer.MIN_VALUE) - 1) * 100);
	private static final BigMoney GBP_LONG_MAX_PLUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE));
	private static final BigMoney GBP_LONG_MIN_MINUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE));
	private static final BigMoney GBP_LONG_MAX_MAJOR_PLUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE).multiply(BigDecimal.valueOf(100)));
	private static final BigMoney GBP_LONG_MIN_MAJOR_MINUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE).multiply(BigDecimal.valueOf(100)));
	private static final BigMoney JPY_423 = BigMoney.parse("JPY 423");
	private static final BigMoney USD_1_23 = BigMoney.parse("USD 1.23");
	private static final BigMoney USD_2_34 = BigMoney.parse("USD 2.34");
	private static final BigMoney USD_2_35 = BigMoney.parse("USD 2.35");
	private static final BigMoneyProvider BAD_PROVIDER = new BigMoneyProvider() {
		@Override
		public BigMoney toBigMoney() {
			return null; // shouldn't return null
		}
	};

	private static BigDecimal bd(String str) {
		return new BigDecimal(str);
	}

	// -----------------------------------------------------------------------
	// plus(BigMoneyProvider)
	// -----------------------------------------------------------------------
	public void test_plus_BigMoneyProvider_zero() {
		BigMoney test = GBP_2_34.plus(GBP_0_00);
	}

	public void test_plus_BigMoneyProvider_positive() {
		BigMoney test = GBP_2_34.plus(GBP_1_23);
	}

	public void test_plus_BigMoneyProvider_negative() {
		BigMoney test = GBP_2_34.plus(GBP_M1_23);
	}

	public void test_plus_BigMoneyProvider_scale() {
		BigMoney test = GBP_2_34.plus(BigMoney.parse("GBP 1.111"));
	}

	public void test_plus_BigMoneyProvider_Money() {
		BigMoney test = GBP_2_34.plus(BigMoney.ofMinor(GBP, 1));
	}

	public void test_plus_BigMoneyProvider_currencyMismatch() {
		GBP_M5_78.plus(USD_1_23);
	}

	public void test_plus_BigMoneyProvider_nullBigMoneyProvider() {
		GBP_M5_78.plus((BigMoneyProvider) null);
	}

	public void test_plus_BigMoneyProvider_badProvider() {
		GBP_M5_78.plus(new BigMoneyProvider() {
			@Override
			public BigMoney toBigMoney() {
				return null;
			}
		});
	}
	
	public static void main(String[] args) {
		TestBigMoney27 t = new TestBigMoney27();
		t.test_plus_BigMoneyProvider_zero();
		t.test_plus_BigMoneyProvider_positive();
		t.test_plus_BigMoneyProvider_negative();
		t.test_plus_BigMoneyProvider_scale();
		t.test_plus_BigMoneyProvider_Money();
	}

}
