package org.joda.money;

import static org.testng.Assert.assertEquals;

import java.math.BigDecimal;

import org.testng.annotations.Test;

import junit.framework.TestCase;

public class TestBigMoney49 extends TestCase {

	private static final CurrencyUnit GBP = CurrencyUnit.of("GBP");
	private static final CurrencyUnit EUR = CurrencyUnit.of("EUR");
	private static final CurrencyUnit USD = CurrencyUnit.of("USD");
	private static final CurrencyUnit JPY = CurrencyUnit.of("JPY");
	private static final BigDecimal BIGDEC_2_34 = new BigDecimal("2.34");
	private static final BigDecimal BIGDEC_2_345 = new BigDecimal("2.345");
	private static final BigDecimal BIGDEC_M5_78 = new BigDecimal("-5.78");

	private static final BigMoney GBP_0_00 = BigMoney.parse("GBP 0.00");
	private static final BigMoney GBP_1_23 = BigMoney.parse("GBP 1.23");
	private static final BigMoney GBP_2_33 = BigMoney.parse("GBP 2.33");
	private static final BigMoney GBP_2_34 = BigMoney.parse("GBP 2.34");
	private static final BigMoney GBP_2_35 = BigMoney.parse("GBP 2.35");
	private static final BigMoney GBP_2_36 = BigMoney.parse("GBP 2.36");
	private static final BigMoney GBP_5_78 = BigMoney.parse("GBP 5.78");
	private static final BigMoney GBP_M1_23 = BigMoney.parse("GBP -1.23");
	private static final BigMoney GBP_M5_78 = BigMoney.parse("GBP -5.78");
	private static final BigMoney GBP_INT_MAX_PLUS1 = BigMoney.ofMinor(GBP, ((long) Integer.MAX_VALUE) + 1);
	private static final BigMoney GBP_INT_MIN_MINUS1 = BigMoney.ofMinor(GBP, ((long) Integer.MIN_VALUE) - 1);
	private static final BigMoney GBP_INT_MAX_MAJOR_PLUS1 = BigMoney.ofMinor(GBP,
			(((long) Integer.MAX_VALUE) + 1) * 100);
	private static final BigMoney GBP_INT_MIN_MAJOR_MINUS1 = BigMoney.ofMinor(GBP,
			(((long) Integer.MIN_VALUE) - 1) * 100);
	private static final BigMoney GBP_LONG_MAX_PLUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE));
	private static final BigMoney GBP_LONG_MIN_MINUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE));
	private static final BigMoney GBP_LONG_MAX_MAJOR_PLUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE).multiply(BigDecimal.valueOf(100)));
	private static final BigMoney GBP_LONG_MIN_MAJOR_MINUS1 = BigMoney.of(GBP,
			BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE).multiply(BigDecimal.valueOf(100)));
	private static final BigMoney JPY_423 = BigMoney.parse("JPY 423");
	private static final BigMoney USD_1_23 = BigMoney.parse("USD 1.23");
	private static final BigMoney USD_2_34 = BigMoney.parse("USD 2.34");
	private static final BigMoney USD_2_35 = BigMoney.parse("USD 2.35");
	private static final BigMoneyProvider BAD_PROVIDER = new BigMoneyProvider() {
		@Override
		public BigMoney toBigMoney() {
			return null; // shouldn't return null
		}
	};

	private static BigDecimal bd(String str) {
		return new BigDecimal(str);
	}

	// -----------------------------------------------------------------------
	// isLessThan()
	// -----------------------------------------------------------------------
	public void test_isLessThan() {
		BigMoney a = GBP_2_34;
		BigMoney b = GBP_2_35;
		BigMoney c = GBP_2_36;
		a.isLessThan(a);
//		assertEquals(a.isLessThan(a), false);
//		assertEquals(b.isLessThan(b), false);
//		assertEquals(c.isLessThan(c), false);
//
//		assertEquals(a.isLessThan(b), true);
//		assertEquals(b.isLessThan(a), false);
//
//		assertEquals(a.isLessThan(c), true);
//		assertEquals(c.isLessThan(a), false);
//
//		assertEquals(b.isLessThan(c), true);
//		assertEquals(c.isLessThan(b), false);
	}
	
	public void test2() {
		BigMoney a = GBP_2_34;
		BigMoney b = GBP_2_35;
		BigMoney c = GBP_2_36;
		b.isLessThan(b);
	}
	
	public void test3() {
		BigMoney a = GBP_2_34;
		BigMoney b = GBP_2_35;
		BigMoney c = GBP_2_36;
		c.isLessThan(c);
	}
	
	public void test4() {
		BigMoney a = GBP_2_34;
		BigMoney b = GBP_2_35;
		BigMoney c = GBP_2_36;
		a.isLessThan(b);
	}
	
	public void test5() {
		BigMoney a = GBP_2_34;
		BigMoney b = GBP_2_35;
		BigMoney c = GBP_2_36;
		b.isLessThan(a);
	}
	
	public void test6() {
		BigMoney a = GBP_2_34;
		BigMoney b = GBP_2_35;
		BigMoney c = GBP_2_36;
		a.isLessThan(c);
	}
	
	public void test7() {
		BigMoney a = GBP_2_34;
		BigMoney b = GBP_2_35;
		BigMoney c = GBP_2_36;
		c.isLessThan(a);
	}
	
	public void test8() {
		BigMoney a = GBP_2_34;
		BigMoney b = GBP_2_35;
		BigMoney c = GBP_2_36;
		b.isLessThan(c);
	}
	
	public void test9() {
		BigMoney a = GBP_2_34;
		BigMoney b = GBP_2_35;
		BigMoney c = GBP_2_36;
		c.isLessThan(b);
	}

	@Test(expectedExceptions = CurrencyMismatchException.class)
	public void test_isLessThan_currenciesDiffer() {
		BigMoney a = GBP_2_34;
		BigMoney b = USD_2_35;
		a.isLessThan(b);
	}
	
	public static void main(String[] args) {
		TestBigMoney49 t = new TestBigMoney49();
		t.test_isLessThan();
		t.test2();
		t.test3();
		t.test4();
		t.test5();
		t.test6();
		t.test7();
		t.test8();
		t.test9();
	}

}
