package org.joda.money;

import java.math.BigDecimal;

import org.testng.annotations.Test;

import junit.framework.TestCase;

public class TestMoney10 extends TestCase {

	private static final CurrencyUnit GBP = CurrencyUnit.of("GBP");
	private static final CurrencyUnit EUR = CurrencyUnit.of("EUR");
	private static final CurrencyUnit USD = CurrencyUnit.of("USD");
	private static final CurrencyUnit JPY = CurrencyUnit.of("JPY");
	private static final BigDecimal BIGDEC_2_3 = new BigDecimal("2.3");
	private static final BigDecimal BIGDEC_2_34 = new BigDecimal("2.34");
	private static final BigDecimal BIGDEC_2_345 = new BigDecimal("2.345");
	private static final BigDecimal BIGDEC_M5_78 = new BigDecimal("-5.78");

	private static final Money GBP_0_00 = Money.parse("GBP 0.00");
	private static final Money GBP_1_23 = Money.parse("GBP 1.23");
	private static final Money GBP_2_33 = Money.parse("GBP 2.33");
	private static final Money GBP_2_34 = Money.parse("GBP 2.34");
	private static final Money GBP_2_35 = Money.parse("GBP 2.35");
	private static final Money GBP_2_36 = Money.parse("GBP 2.36");
	private static final Money GBP_5_78 = Money.parse("GBP 5.78");
	private static final Money GBP_M1_23 = Money.parse("GBP -1.23");
	private static final Money GBP_M5_78 = Money.parse("GBP -5.78");
	private static final Money GBP_INT_MAX_PLUS1 = Money.ofMinor(GBP, ((long) Integer.MAX_VALUE) + 1);
	private static final Money GBP_INT_MIN_MINUS1 = Money.ofMinor(GBP, ((long) Integer.MIN_VALUE) - 1);
	private static final Money GBP_INT_MAX_MAJOR_PLUS1 = Money.ofMinor(GBP, (((long) Integer.MAX_VALUE) + 1) * 100);
	private static final Money GBP_INT_MIN_MAJOR_MINUS1 = Money.ofMinor(GBP, (((long) Integer.MIN_VALUE) - 1) * 100);
	private static final Money GBP_LONG_MAX_PLUS1 = Money.of(GBP,
			BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE));
	private static final Money GBP_LONG_MIN_MINUS1 = Money.of(GBP,
			BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE));
	private static final Money GBP_LONG_MAX_MAJOR_PLUS1 = Money.of(GBP,
			BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE).multiply(BigDecimal.valueOf(100)));
	private static final Money GBP_LONG_MIN_MAJOR_MINUS1 = Money.of(GBP,
			BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE).multiply(BigDecimal.valueOf(100)));
	private static final Money JPY_423 = Money.parse("JPY 423");
	private static final Money USD_1_23 = Money.parse("USD 1.23");
	private static final Money USD_2_34 = Money.parse("USD 2.34");
	private static final Money USD_2_35 = Money.parse("USD 2.35");

	// -----------------------------------------------------------------------
	// total(Money...)
	// -----------------------------------------------------------------------
	public void test_factory_total_varargs_1() {
		Money test = Money.total(GBP_1_23);
	}

	public void test_factory_total_array_1() {
		Money[] array = new Money[] { GBP_1_23 };
		Money test = Money.total(array);
	}

	public void test_factory_total_varargs_3() {
		Money test = Money.total(GBP_1_23, GBP_2_33, GBP_2_36);
	}

	public void test_factory_total_array_3() {
		Money[] array = new Money[] { GBP_1_23, GBP_2_33, GBP_2_36 };
		Money test = Money.total(array);
	}

	@Test(expectedExceptions = IllegalArgumentException.class)
	public void test_factory_total_varargs_empty() {
		Money.total();
	}

	@Test(expectedExceptions = IllegalArgumentException.class)
	public void test_factory_total_array_empty() {
		Money[] array = new Money[0];
		Money.total(array);
	}

	@Test(expectedExceptions = CurrencyMismatchException.class)
	public void test_factory_total_varargs_currenciesDiffer() {
		Money.total(GBP_2_33, JPY_423);
	}

	@Test(expectedExceptions = CurrencyMismatchException.class)
	public void test_factory_total_array_currenciesDiffer() {
		Money[] array = new Money[] { GBP_2_33, JPY_423 };
		Money.total(array);
	}

	@Test(expectedExceptions = NullPointerException.class)
	public void test_factory_total_varargs_nullFirst() {
		Money.total((Money) null, GBP_2_33, GBP_2_36);
	}

	@Test(expectedExceptions = NullPointerException.class)
	public void test_factory_total_array_nullFirst() {
		Money[] array = new Money[] { null, GBP_2_33, GBP_2_36 };
		Money.total(array);
	}

	@Test(expectedExceptions = NullPointerException.class)
	public void test_factory_total_varargs_nullNotFirst() {
		Money.total(GBP_2_33, null, GBP_2_36);
	}

	@Test(expectedExceptions = NullPointerException.class)
	public void test_factory_total_array_nullNotFirst() {
		Money[] array = new Money[] { GBP_2_33, null, GBP_2_36 };
		Money.total(array);
	}
	
	public static void main(String[] args) {
		TestMoney10 t = new TestMoney10();
		t.test_factory_total_varargs_1();
		t.test_factory_total_array_1();
		t.test_factory_total_varargs_3();
		t.test_factory_total_array_3();
	}

}
