package org.joda.money;

import static org.testng.Assert.assertEquals;

import java.math.BigDecimal;

import org.testng.annotations.Test;

import junit.framework.TestCase;

public class TestMoney46 extends TestCase {

	private static final CurrencyUnit GBP = CurrencyUnit.of("GBP");
	private static final CurrencyUnit EUR = CurrencyUnit.of("EUR");
	private static final CurrencyUnit USD = CurrencyUnit.of("USD");
	private static final CurrencyUnit JPY = CurrencyUnit.of("JPY");
	private static final BigDecimal BIGDEC_2_3 = new BigDecimal("2.3");
	private static final BigDecimal BIGDEC_2_34 = new BigDecimal("2.34");
	private static final BigDecimal BIGDEC_2_345 = new BigDecimal("2.345");
	private static final BigDecimal BIGDEC_M5_78 = new BigDecimal("-5.78");

	private static final Money GBP_0_00 = Money.parse("GBP 0.00");
	private static final Money GBP_1_23 = Money.parse("GBP 1.23");
	private static final Money GBP_2_33 = Money.parse("GBP 2.33");
	private static final Money GBP_2_34 = Money.parse("GBP 2.34");
	private static final Money GBP_2_35 = Money.parse("GBP 2.35");
	private static final Money GBP_2_36 = Money.parse("GBP 2.36");
	private static final Money GBP_5_78 = Money.parse("GBP 5.78");
	private static final Money GBP_M1_23 = Money.parse("GBP -1.23");
	private static final Money GBP_M5_78 = Money.parse("GBP -5.78");
	private static final Money GBP_INT_MAX_PLUS1 = Money.ofMinor(GBP, ((long) Integer.MAX_VALUE) + 1);
	private static final Money GBP_INT_MIN_MINUS1 = Money.ofMinor(GBP, ((long) Integer.MIN_VALUE) - 1);
	private static final Money GBP_INT_MAX_MAJOR_PLUS1 = Money.ofMinor(GBP, (((long) Integer.MAX_VALUE) + 1) * 100);
	private static final Money GBP_INT_MIN_MAJOR_MINUS1 = Money.ofMinor(GBP, (((long) Integer.MIN_VALUE) - 1) * 100);
	private static final Money GBP_LONG_MAX_PLUS1 = Money.of(GBP,
			BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE));
	private static final Money GBP_LONG_MIN_MINUS1 = Money.of(GBP,
			BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE));
	private static final Money GBP_LONG_MAX_MAJOR_PLUS1 = Money.of(GBP,
			BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE).multiply(BigDecimal.valueOf(100)));
	private static final Money GBP_LONG_MIN_MAJOR_MINUS1 = Money.of(GBP,
			BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE).multiply(BigDecimal.valueOf(100)));
	private static final Money JPY_423 = Money.parse("JPY 423");
	private static final Money USD_1_23 = Money.parse("USD 1.23");
	private static final Money USD_2_34 = Money.parse("USD 2.34");
	private static final Money USD_2_35 = Money.parse("USD 2.35");

	// -----------------------------------------------------------------------
	// isEqual()
	// -----------------------------------------------------------------------
	public void test_isEqual() {
		Money a = GBP_2_34;
		Money b = GBP_2_35;
		Money c = GBP_2_36;
		a.isEqual(a);
//		assertEquals(a.isEqual(a), true);
//		assertEquals(b.isEqual(b), true);
//		assertEquals(c.isEqual(c), true);
//
//		assertEquals(a.isEqual(b), false);
//		assertEquals(b.isEqual(a), false);
//
//		assertEquals(a.isEqual(c), false);
//		assertEquals(c.isEqual(a), false);
//
//		assertEquals(b.isEqual(c), false);
//		assertEquals(c.isEqual(b), false);
	}
	
	public void test2() {
		Money a = GBP_2_34;
		Money b = GBP_2_35;
		Money c = GBP_2_36;
		b.isEqual(b);
	}
	
	public void test3() {
		Money a = GBP_2_34;
		Money b = GBP_2_35;
		Money c = GBP_2_36;
		c.isEqual(c);
	}
	
	public void test4() {
		Money a = GBP_2_34;
		Money b = GBP_2_35;
		Money c = GBP_2_36;
		a.isEqual(b);
	}
	
	public void test5() {
		Money a = GBP_2_34;
		Money b = GBP_2_35;
		Money c = GBP_2_36;
		b.isEqual(a);
	}
	
	public void test6() {
		Money a = GBP_2_34;
		Money b = GBP_2_35;
		Money c = GBP_2_36;
		a.isEqual(c);
	}
	
	public void test7() {
		Money a = GBP_2_34;
		Money b = GBP_2_35;
		Money c = GBP_2_36;
		c.isEqual(a);
	}
	
	public void test8() {
		Money a = GBP_2_34;
		Money b = GBP_2_35;
		Money c = GBP_2_36;
		b.isEqual(c);
	}
	
	public void test9() {
		Money a = GBP_2_34;
		Money b = GBP_2_35;
		Money c = GBP_2_36;
		c.isEqual(b);
	}

	public void test_isEqual_Money() {
		Money a = GBP_2_34;
		BigMoney b = BigMoney.ofMinor(GBP, 234);
		a.isEqual(b);
//		assertEquals(a.isEqual(b), true);
	}

	@Test(expectedExceptions = CurrencyMismatchException.class)
	public void test_isEqual_currenciesDiffer() {
		Money a = GBP_2_34;
		Money b = USD_2_35;
		a.isEqual(b);
	}
	
	public static void main(String[] args) {
		TestMoney46 t = new TestMoney46();
		t.test_isEqual();
		t.test_isEqual_Money();
		t.test2();
		t.test3();
		t.test4();
		t.test5();
		t.test6();
		t.test7();
		t.test8();
		t.test9();
	}

}
