package org.joda.time;

import org.joda.time.chrono.BuddhistChronology;
import org.joda.time.chrono.CopticChronology;
import org.joda.time.chrono.GJChronology;
import org.joda.time.chrono.GregorianChronology;
import org.joda.time.chrono.ISOChronology;

import junit.framework.TestCase;

public class TestLocalDateTime_Basics12 extends TestCase {

	private static final DateTimeZone PARIS = DateTimeZone.forID("Europe/Paris");
	private static final DateTimeZone LONDON = DateTimeZone.forID("Europe/London");
	private static final DateTimeZone TOKYO = DateTimeZone.forID("Asia/Tokyo");
	private static final DateTimeZone NEW_YORK = DateTimeZone.forID("America/New_York");
	private static final GJChronology GJ_UTC = GJChronology.getInstanceUTC();
	private static final Chronology COPTIC_PARIS = CopticChronology.getInstance(PARIS);
	private static final Chronology COPTIC_LONDON = CopticChronology.getInstance(LONDON);
	private static final Chronology COPTIC_TOKYO = CopticChronology.getInstance(TOKYO);
	private static final Chronology COPTIC_UTC = CopticChronology.getInstanceUTC();
	private static final Chronology ISO_LONDON = ISOChronology.getInstance(LONDON);
	private static final Chronology ISO_NEW_YORK = ISOChronology.getInstance(NEW_YORK);
	private static final Chronology ISO_UTC = ISOChronology.getInstanceUTC();
	private static final Chronology GREGORIAN_UTC = GregorianChronology.getInstanceUTC();
	private static final Chronology BUDDHIST_LONDON = BuddhistChronology.getInstance(LONDON);
	private static final Chronology BUDDHIST_TOKYO = BuddhistChronology.getInstance(TOKYO);

	public void testToDateTime_Zone() {
		LocalDateTime base = new LocalDateTime(2005, 6, 9, 6, 7, 8, 9, COPTIC_PARIS); // PARIS
																						// irrelevant
		DateTime test = base.toDateTime(TOKYO);
	}

	public void testToDateTime_nullZone() {
		LocalDateTime base = new LocalDateTime(2005, 6, 9, 6, 7, 8, 9, COPTIC_PARIS); // PARIS
																						// irrelevant
		DateTime test = base.toDateTime((DateTimeZone) null);
	}

	public void testToDateTime_Zone_dstGap() {
		LocalDateTime base = new LocalDateTime(2014, 3, 30, 1, 30, 0, 0, ISO_LONDON);
		base.toDateTime(LONDON);
	}

	public void testToDateTime_Zone_dstOverlap() {
		LocalDateTime base = new LocalDateTime(2014, 10, 26, 1, 30, 0, 0, ISO_LONDON);
		DateTime test = base.toDateTime(LONDON);
	}

	public void testToDateTime_Zone_dstOverlap_NewYork() {
		LocalDateTime base = new LocalDateTime(2007, 11, 4, 1, 30, 0, 0, ISO_NEW_YORK);
		DateTime test = base.toDateTime(NEW_YORK);
	}
	
	public static void main(String[] args) throws Throwable {
		TestLocalDateTime_Basics12 t = new TestLocalDateTime_Basics12();
		t.testToDateTime_Zone();
		t.testToDateTime_nullZone();
		t.testToDateTime_Zone_dstOverlap();
		t.testToDateTime_Zone_dstOverlap_NewYork();
	}

}
