package org.joda.time;

import org.joda.time.chrono.BuddhistChronology;
import org.joda.time.chrono.CopticChronology;
import org.joda.time.chrono.ISOChronology;

import junit.framework.TestCase;

public class TestLocalDate_Basics13 extends TestCase {

	private static final DateTimeZone PARIS = DateTimeZone.forID("Europe/Paris");
	private static final DateTimeZone TOKYO = DateTimeZone.forID("Asia/Tokyo");
	private static final DateTimeZone LONDON = DateTimeZone.forID("Europe/London");
	private static final DateTimeZone NEW_YORK = DateTimeZone.forID("America/New_York");

	private static final Chronology COPTIC_PARIS = CopticChronology.getInstance(PARIS);
	private static final Chronology COPTIC_TOKYO = CopticChronology.getInstance(TOKYO);
	private static final Chronology ISO_LONDON = ISOChronology.getInstance(LONDON);
	private static final Chronology ISO_NEW_YORK = ISOChronology.getInstance(NEW_YORK);
	private static final Chronology BUDDHIST_TOKYO = BuddhistChronology.getInstance(TOKYO);

	public void testToDateTime_LocalTime_Zone() {
		LocalDate base = new LocalDate(2005, 6, 9, COPTIC_PARIS); // PARIS
																	// irrelevant
		LocalTime tod = new LocalTime(12, 13, 14, 15, COPTIC_TOKYO);
		DateTime test = base.toDateTime(tod, TOKYO);
	}

	public void testToDateTime_LocalTime_nullZone() {
		LocalDate base = new LocalDate(2005, 6, 9, COPTIC_PARIS); // PARIS
																	// irrelevant
		LocalTime tod = new LocalTime(12, 13, 14, 15, COPTIC_TOKYO);
		DateTime test = base.toDateTime(tod, null);
	}

	public void testToDateTime_nullLocalTime_Zone() {
		LocalDate base = new LocalDate(2005, 6, 9, COPTIC_PARIS); // PARIS
																	// irrelevant
		long now = new DateTime(2004, 5, 8, 12, 13, 14, 15, COPTIC_TOKYO).getMillis();
		DateTimeUtils.setCurrentMillisFixed(now);
		DateTime test = base.toDateTime((LocalTime) null, TOKYO);
	}

	public void testToDateTime_LocalTime_Zone_dstGap() {
		LocalDate base = new LocalDate(2014, 3, 30, ISO_LONDON);
		LocalTime tod = new LocalTime(1, 30, 0, 0, ISO_LONDON);
		base.toDateTime(tod, LONDON);
	}

	public void testToDateTime_LocalTime_Zone_dstOverlap() {
		LocalDate base = new LocalDate(2014, 10, 26, ISO_LONDON);
		LocalTime tod = new LocalTime(1, 30, 0, 0, ISO_LONDON);
		DateTime test = base.toDateTime(tod, LONDON);
	}

	public void testToDateTime_LocalTime_Zone_dstOverlap_NewYork() {
		LocalDate base = new LocalDate(2007, 11, 4, ISO_NEW_YORK);
		LocalTime tod = new LocalTime(1, 30, 0, 0, ISO_NEW_YORK);
		DateTime test = base.toDateTime(tod, NEW_YORK);
	}

	public void testToDateTime_wrongChronoLocalTime_Zone() {
		LocalDate base = new LocalDate(2005, 6, 9, COPTIC_PARIS); // PARIS
																	// irrelevant
		LocalTime tod = new LocalTime(12, 13, 14, 15, BUDDHIST_TOKYO);
		base.toDateTime(tod, LONDON);
	}
	
	public static void main(String[] args) throws Throwable {
		TestLocalDate_Basics13 t = new TestLocalDate_Basics13();
		t.testToDateTime_LocalTime_Zone();
		t.testToDateTime_LocalTime_nullZone();
		t.testToDateTime_nullLocalTime_Zone();
		t.testToDateTime_LocalTime_Zone_dstOverlap();
		t.testToDateTime_LocalTime_Zone_dstOverlap_NewYork();
	}

}
