package org.joda.time;

import org.joda.time.chrono.BuddhistChronology;
import org.joda.time.chrono.CopticChronology;
import org.joda.time.chrono.ISOChronology;

import junit.framework.TestCase;

public class TestPartial_Basics12 extends TestCase {

	private static final DateTimeZone PARIS = DateTimeZone.forID("Europe/Paris");
	private static final DateTimeZone LONDON = DateTimeZone.forID("Europe/London");
	private static final DateTimeZone TOKYO = DateTimeZone.forID("Asia/Tokyo");
	private static final Chronology COPTIC_PARIS = CopticChronology.getInstance(PARIS);
	private static final Chronology COPTIC_TOKYO = CopticChronology.getInstance(TOKYO);
	private static final Chronology COPTIC_UTC = CopticChronology.getInstanceUTC();
	private static final Chronology ISO_UTC = ISOChronology.getInstanceUTC();
	private static final Chronology BUDDHIST_LONDON = BuddhistChronology.getInstance(LONDON);
	private static final Chronology BUDDHIST_TOKYO = BuddhistChronology.getInstance(TOKYO);
	private static final Chronology BUDDHIST_UTC = BuddhistChronology.getInstanceUTC();

	private long TEST_TIME_NOW = 10L * DateTimeConstants.MILLIS_PER_HOUR + 20L * DateTimeConstants.MILLIS_PER_MINUTE
			+ 30L * DateTimeConstants.MILLIS_PER_SECOND + 40L;
	private long TEST_TIME2 = 1L * DateTimeConstants.MILLIS_PER_DAY + 5L * DateTimeConstants.MILLIS_PER_HOUR
			+ 6L * DateTimeConstants.MILLIS_PER_MINUTE + 7L * DateTimeConstants.MILLIS_PER_SECOND + 8L;

	private Partial createHourMinPartial() {
		return createHourMinPartial(ISO_UTC);
	}

	private Partial createHourMinPartial(Chronology chrono) {
		return createHourMinPartial(10, 20, chrono);
	}

	private Partial createHourMinPartial2(Chronology chrono) {
		return createHourMinPartial(15, 20, chrono);
	}

	private Partial createHourMinPartial(int hour, int min, Chronology chrono) {
		return new Partial(new DateTimeFieldType[] { DateTimeFieldType.hourOfDay(), DateTimeFieldType.minuteOfHour() },
				new int[] { hour, min }, chrono);
	}

	private Partial createTODPartial(Chronology chrono) {
		return new Partial(
				new DateTimeFieldType[] { DateTimeFieldType.hourOfDay(), DateTimeFieldType.minuteOfHour(),
						DateTimeFieldType.secondOfMinute(), DateTimeFieldType.millisOfSecond() },
				new int[] { 10, 20, 30, 40 }, chrono);
	}

	public void testWithFieldAddWrapped1() {
		Partial test = createHourMinPartial();
		Partial result = test.withFieldAddWrapped(DurationFieldType.hours(), 6);
	}

	public void testWithFieldAddWrapped2() {
		Partial test = createHourMinPartial();
		test.withFieldAddWrapped(null, 0);
	}

	public void testWithFieldAddWrapped3() {
		Partial test = createHourMinPartial();
		test.withFieldAddWrapped(null, 6);
	}

	public void testWithFieldAddWrapped4() {
		Partial test = createHourMinPartial();
		Partial result = test.withFieldAddWrapped(DurationFieldType.hours(), 0);
	}

	public void testWithFieldAddWrapped5() {
		Partial test = createHourMinPartial();
		test.withFieldAddWrapped(DurationFieldType.days(), 6);
	}

	public void testWithFieldAddWrapped6() {
		Partial test = createHourMinPartial();
		Partial result = test.withFieldAddWrapped(DurationFieldType.hours(), 16);
	}

	public void testWithFieldAddWrapped7() {
		Partial test = createHourMinPartial(23, 59, ISO_UTC);
		Partial result = test.withFieldAddWrapped(DurationFieldType.minutes(), 1);
	}

	public void test7b() {
		Partial test = createHourMinPartial(23, 59, ISO_UTC);
		Partial result = test.withFieldAddWrapped(DurationFieldType.hours(), 1);
	}

	public void testWithFieldAddWrapped8() {
		Partial test = createHourMinPartial(0, 0, ISO_UTC);
		Partial result = test.withFieldAddWrapped(DurationFieldType.minutes(), -1);
	}

	public void test8b() {
		Partial test = createHourMinPartial(0, 0, ISO_UTC);
		Partial result = test.withFieldAddWrapped(DurationFieldType.hours(), -1);
	}
	
	public static void main(String[] args) {
		TestPartial_Basics12 t = new TestPartial_Basics12();
		t.testWithFieldAddWrapped1();
		t.testWithFieldAddWrapped4();
		t.testWithFieldAddWrapped6();
		t.testWithFieldAddWrapped7();
		t.testWithFieldAddWrapped8();
		t.test7b();
		t.test8b();
	}

}
