package org.joda.time;

import org.joda.time.chrono.BuddhistChronology;
import org.joda.time.chrono.CopticChronology;
import org.joda.time.chrono.ISOChronology;

import junit.framework.TestCase;

public class TestPartial_Basics9 extends TestCase {

	private static final DateTimeZone PARIS = DateTimeZone.forID("Europe/Paris");
	private static final DateTimeZone LONDON = DateTimeZone.forID("Europe/London");
	private static final DateTimeZone TOKYO = DateTimeZone.forID("Asia/Tokyo");
	private static final Chronology COPTIC_PARIS = CopticChronology.getInstance(PARIS);
	private static final Chronology COPTIC_TOKYO = CopticChronology.getInstance(TOKYO);
	private static final Chronology COPTIC_UTC = CopticChronology.getInstanceUTC();
	private static final Chronology ISO_UTC = ISOChronology.getInstanceUTC();
	private static final Chronology BUDDHIST_LONDON = BuddhistChronology.getInstance(LONDON);
	private static final Chronology BUDDHIST_TOKYO = BuddhistChronology.getInstance(TOKYO);
	private static final Chronology BUDDHIST_UTC = BuddhistChronology.getInstanceUTC();

	private long TEST_TIME_NOW = 10L * DateTimeConstants.MILLIS_PER_HOUR + 20L * DateTimeConstants.MILLIS_PER_MINUTE
			+ 30L * DateTimeConstants.MILLIS_PER_SECOND + 40L;
	private long TEST_TIME2 = 1L * DateTimeConstants.MILLIS_PER_DAY + 5L * DateTimeConstants.MILLIS_PER_HOUR
			+ 6L * DateTimeConstants.MILLIS_PER_MINUTE + 7L * DateTimeConstants.MILLIS_PER_SECOND + 8L;

	private Partial createHourMinPartial() {
		return createHourMinPartial(ISO_UTC);
	}

	private Partial createHourMinPartial(Chronology chrono) {
		return createHourMinPartial(10, 20, chrono);
	}

	private Partial createHourMinPartial2(Chronology chrono) {
		return createHourMinPartial(15, 20, chrono);
	}

	private Partial createHourMinPartial(int hour, int min, Chronology chrono) {
		return new Partial(new DateTimeFieldType[] { DateTimeFieldType.hourOfDay(), DateTimeFieldType.minuteOfHour() },
				new int[] { hour, min }, chrono);
	}

	private Partial createTODPartial(Chronology chrono) {
		return new Partial(
				new DateTimeFieldType[] { DateTimeFieldType.hourOfDay(), DateTimeFieldType.minuteOfHour(),
						DateTimeFieldType.secondOfMinute(), DateTimeFieldType.millisOfSecond() },
				new int[] { 10, 20, 30, 40 }, chrono);
	}

	public void testWith1() {
		Partial test = createHourMinPartial();
		Partial result = test.with(DateTimeFieldType.hourOfDay(), 15);
	}

	public void testWith2() {
		Partial test = createHourMinPartial();
		test.with(null, 6);
	}

	public void testWith3() {
		Partial test = createHourMinPartial();
		test.with(DateTimeFieldType.clockhourOfDay(), 6);
	}

	public void testWith3a() {
		Partial test = createHourMinPartial();
		Partial result = test.with(DateTimeFieldType.secondOfMinute(), 15);
	}

	public void testWith3b() {
		Partial test = createHourMinPartial();
		Partial result = test.with(DateTimeFieldType.minuteOfDay(), 15);
	}

	public void testWith3c() {
		Partial test = createHourMinPartial();
		Partial result = test.with(DateTimeFieldType.dayOfMonth(), 15);
	}

	public void testWith3d() {
		Partial test = new Partial(DateTimeFieldType.year(), 2005);
		Partial result = test.with(DateTimeFieldType.monthOfYear(), 6);
	}

	public void testWith3e() {
		Partial test = new Partial(DateTimeFieldType.era(), 1);
		Partial result = test.with(DateTimeFieldType.halfdayOfDay(), 0);
	}

	public void testWith3f() {
		Partial test = new Partial(DateTimeFieldType.halfdayOfDay(), 0);
		Partial result = test.with(DateTimeFieldType.era(), 1);
	}

	public void testWith4() {
		Partial test = createHourMinPartial();
		Partial result = test.with(DateTimeFieldType.hourOfDay(), 10);
	}

	public void testWith_baseHasNoRange() {
		Partial test = new Partial(DateTimeFieldType.year(), 1);
		Partial result = test.with(DateTimeFieldType.hourOfDay(), 10);
	}

	public void testWith_argHasNoRange() {
		Partial test = new Partial(DateTimeFieldType.hourOfDay(), 1);
		Partial result = test.with(DateTimeFieldType.year(), 10);
	}

	public void testWith_baseAndArgHaveNoRange() {
		Partial test = new Partial(DateTimeFieldType.year(), 1);
		Partial result = test.with(DateTimeFieldType.era(), 1);
	}
	
	public static void main(String[] args) {
		TestPartial_Basics9 t = new TestPartial_Basics9();
		t.testWith1();
		t.testWith4();
		t.testWith3a();
		t.testWith3b();
		t.testWith3c();
		t.testWith3d();
		t.testWith3e();
		t.testWith3f();
		t.testWith_baseHasNoRange();
		t.testWith_argHasNoRange();
		t.testWith_baseAndArgHaveNoRange();
	}

}
