package org.joda.time;

import org.joda.time.chrono.BuddhistChronology;
import org.joda.time.chrono.CopticChronology;
import org.joda.time.chrono.ISOChronology;

import junit.framework.TestCase;

public class TestYearMonthDay_Basics9 extends TestCase {

	private static final DateTimeZone PARIS = DateTimeZone.forID("Europe/Paris");
	private static final DateTimeZone LONDON = DateTimeZone.forID("Europe/London");
	private static final DateTimeZone TOKYO = DateTimeZone.forID("Asia/Tokyo");
	private static final Chronology COPTIC_PARIS = CopticChronology.getInstance(PARIS);
	private static final Chronology COPTIC_LONDON = CopticChronology.getInstance(LONDON);
	private static final Chronology COPTIC_TOKYO = CopticChronology.getInstance(TOKYO);
	private static final Chronology COPTIC_UTC = CopticChronology.getInstanceUTC();
	private static final Chronology ISO_UTC = ISOChronology.getInstanceUTC();
	private static final Chronology BUDDHIST_TOKYO = BuddhistChronology.getInstance(TOKYO);
	private static final Chronology BUDDHIST_UTC = BuddhistChronology.getInstanceUTC();

	private long TEST_TIME_NOW = (31L + 28L + 31L + 30L + 31L + 9L - 1L) * DateTimeConstants.MILLIS_PER_DAY;

	public void testWithChronologyRetainFields_Chrono() {
		YearMonthDay base = new YearMonthDay(2005, 6, 9, COPTIC_PARIS);
		YearMonthDay test = base.withChronologyRetainFields(BUDDHIST_TOKYO);
	}

	public void testWithChronologyRetainFields_sameChrono() {
		YearMonthDay base = new YearMonthDay(2005, 6, 9, COPTIC_PARIS);
		YearMonthDay test = base.withChronologyRetainFields(COPTIC_TOKYO);
	}

	public void testWithChronologyRetainFields_nullChrono() {
		YearMonthDay base = new YearMonthDay(2005, 6, 9, COPTIC_PARIS);
		YearMonthDay test = base.withChronologyRetainFields(null);
	}

	public void testWithChronologyRetainFields_invalidInNewChrono() {
		YearMonthDay base = new YearMonthDay(2005, 1, 31, ISO_UTC);
		base.withChronologyRetainFields(COPTIC_UTC);
	}
	
	public static void main(String[] args) {
		TestYearMonthDay_Basics9 t = new TestYearMonthDay_Basics9();
		t.testWithChronologyRetainFields_Chrono();
		t.testWithChronologyRetainFields_sameChrono();
		t.testWithChronologyRetainFields_nullChrono();
	}

}
